/*
 * AudioFXDemo.java
 * geschrieben 2010 von Thomas Knneth
 */
package com.thomaskuenneth.artikel.ix;

import android.app.Activity;
import android.media.MediaPlayer;
import android.media.audiofx.AudioEffect;
import android.media.audiofx.BassBoost;
import android.media.audiofx.Virtualizer;
import android.os.Bundle;
import android.util.Log;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;

/**
 * Diese Klasse demonstriert die Verwendung der von {@link AudioEffect}
 * abgeleiteten Klassen {@link Virtualizer} und {@link BassBoost}. Um die Demo
 * ausprobieren zu knnen, muss unter <em>res/raw</em> eine Audiodatei abgelegt
 * werden. Ihr Name ohne Extension wird
 * {@link MediaPlayer#create(android.content.Context, int)} bergeben.
 * 
 * @author Thomas Knneth
 */
public class AudioFXDemo extends Activity {

	private static final String TAG = AudioFXDemo.class.getSimpleName();

	private MediaPlayer mediaPlayer;
	private BassBoost bassBoost;
	private Virtualizer virtualizer;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		// MediaPlayer instantiieren - hier Datei angeben
		mediaPlayer = MediaPlayer.create(this, R.raw.paranoid);
		int sessionId = mediaPlayer.getAudioSessionId();

		// BassBoost instantiieren und an Audio Session binden
		bassBoost = new BassBoost(0, sessionId);
		Log.d(TAG, "getRoundedStrength(): " + bassBoost.getRoundedStrength());
		if (bassBoost.getStrengthSupported()) {
			bassBoost.setStrength((short) 1000);
		}
		// Checkbox schaltet BassBoost aus und ein
		final CheckBox cbBassBoost = (CheckBox) findViewById(R.id.cbBassBoost);
		cbBassBoost.setOnCheckedChangeListener(new OnCheckedChangeListener() {

			public void onCheckedChanged(CompoundButton buttonView,
					boolean isChecked) {
				int result = bassBoost.setEnabled(isChecked);
				if (result != AudioEffect.SUCCESS) {
					Log.e(TAG, "Bass Boost: setEnabled(" + isChecked + ") = "
							+ result);
				}
			}
		});
		cbBassBoost.setChecked(false);

		// Virtualizer instantiieren und an Audio Session binden
		virtualizer = new Virtualizer(0, sessionId);
		virtualizer.setStrength((short) 1000);
		// Checkbox schaltet Virtualizer aus und ein
		final CheckBox cbVirtualizer = (CheckBox) findViewById(R.id.cbReverb);
		cbVirtualizer.setOnCheckedChangeListener(new OnCheckedChangeListener() {

			public void onCheckedChanged(CompoundButton buttonView,
					boolean isChecked) {
				int result = virtualizer.setEnabled(isChecked);
				if (result != AudioEffect.SUCCESS) {
					Log.e(TAG, "Virtualizer: setEnabled(" + isChecked + ") = "
							+ result);
				}
			}
		});
		cbVirtualizer.setChecked(false);

		// Musik abspielen
		mediaPlayer.start();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		mediaPlayer.stop();
		bassBoost.release();
		virtualizer.release();
		mediaPlayer.release();
	}
}