/**
 * DownloadManagerDemo.java
 * 
 * geschrieben 2010 von Thomas Knneth
 */
package com.thomaskuenneth.artikel.ix;

import java.net.URL;
import java.net.URLConnection;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import android.app.Activity;
import android.app.DownloadManager;
import android.app.DownloadManager.Query;
import android.app.DownloadManager.Request;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.util.Xml;
import android.util.Xml.Encoding;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

/**
 * Diese Activity ldt das "Large Image of the day", das die NASA unter
 * http://www.nasa.gov/rss/lg_image_of_the_day.rss zur Verfgung stellt.
 * 
 * @author Thomas Knneth
 * 
 */
public class DownloadManagerDemo extends Activity {

	private static final String TAG = DownloadManagerDemo.class.getSimpleName();
	private static final String RSS_URL = "http://www.nasa.gov/rss/lg_image_of_the_day.rss";

	private static final int IS_NOT_DOWNLOADING = 0;
	private static final int IS_DOWNLOADING = 1;
	private static final int DOWNLOAD_FINISHED = 2;

	private BroadcastReceiver receiver;

	private boolean isItem, isTitle, isDescription;
	private String imageUrl;
	private String title;
	private String description;

	private int buttonMode;
	private long lastDownloadId;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		final DownloadManager dm = (DownloadManager) getSystemService(DOWNLOAD_SERVICE);

		setContentView(R.layout.main);
		TextView tvTitle = (TextView) findViewById(R.id.text_title);
		TextView tvDescr = (TextView) findViewById(R.id.text_descr);
		final Button button = (Button) findViewById(R.id.button);
		button.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				switch (buttonMode) {
				case IS_NOT_DOWNLOADING:
					Request request = new Request(Uri.parse(imageUrl));
					lastDownloadId = dm.enqueue(request);
					buttonMode = IS_DOWNLOADING;
					button.setText(R.string.button_remove);
					break;
				case IS_DOWNLOADING:
					dm.remove(lastDownloadId);
					buttonMode = IS_NOT_DOWNLOADING;
					button.setText(R.string.button_enqueue);
					break;
				case DOWNLOAD_FINISHED:
					Intent i = new Intent();
					i.setAction(DownloadManager.ACTION_VIEW_DOWNLOADS);
					startActivity(i);
					break;
				}
			}
		});

		receiver = new BroadcastReceiver() {

			@Override
			public void onReceive(Context context, Intent intent) {
				String action = intent.getAction();
				if (DownloadManager.ACTION_DOWNLOAD_COMPLETE.equals(action)) {
					long id = intent.getLongExtra(
							DownloadManager.EXTRA_DOWNLOAD_ID, 0);
					int buttonText = R.string.button_enqueue;
					if (lastDownloadId == id) {
						buttonMode = IS_NOT_DOWNLOADING;
						Query query = new Query();
						query.setFilterById(lastDownloadId);
						Cursor c = dm.query(query);
						if (c.moveToFirst()) {
							int columnIndex = c
									.getColumnIndex(DownloadManager.COLUMN_STATUS);
							if (DownloadManager.STATUS_SUCCESSFUL == c
									.getInt(columnIndex)) {
								buttonMode = DOWNLOAD_FINISHED;
								buttonText = R.string.button_open;
							}
						}
						c.close();
						button.setText(buttonText);
					}
				}
			}
		};
		registerReceiver(receiver, new IntentFilter(
				DownloadManager.ACTION_DOWNLOAD_COMPLETE));

		boolean ok = loadAndParse();
		tvTitle.setText(title);
		tvDescr.setText(description);
		button.setEnabled(ok);
		buttonMode = IS_NOT_DOWNLOADING;
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		unregisterReceiver(receiver);
	}

	private boolean loadAndParse() {
		boolean ok = false;
		isItem = false;
		isTitle = false;
		isDescription = false;
		final StringBuilder sbDescription = new StringBuilder();
		final StringBuilder sbTitle = new StringBuilder();
		imageUrl = null;
		try {
			URL url = new URL(RSS_URL);
			URLConnection c = url.openConnection();
			c.connect();
			Xml.parse(c.getInputStream(), Encoding.UTF_8, new ContentHandler() {

				@Override
				public void startPrefixMapping(String prefix, String uri)
						throws SAXException {
				}

				@Override
				public void startElement(String uri, String localName,
						String qName, Attributes atts) throws SAXException {
					if ("item".equalsIgnoreCase(qName)) {
						isItem = true;
					} else if (isItem) {
						if ("enclosure".equalsIgnoreCase(qName)) {
							imageUrl = atts.getValue("url");
						} else if ("title".equalsIgnoreCase(qName)) {
							isTitle = true;
						} else if ("description".equalsIgnoreCase(qName)) {
							isDescription = true;
						}
					}
				}

				@Override
				public void startDocument() throws SAXException {
				}

				@Override
				public void skippedEntity(String name) throws SAXException {
				}

				@Override
				public void setDocumentLocator(Locator locator) {
				}

				@Override
				public void processingInstruction(String target, String data)
						throws SAXException {
				}

				@Override
				public void ignorableWhitespace(char[] ch, int start, int length)
						throws SAXException {
				}

				@Override
				public void endPrefixMapping(String prefix) throws SAXException {
				}

				@Override
				public void endElement(String uri, String localName,
						String qName) throws SAXException {
					if ("item".equalsIgnoreCase(qName)) {
						isItem = false;
					}
					if (isItem) {
						if ("title".equalsIgnoreCase(qName)) {
							isTitle = false;
						} else if ("description".equalsIgnoreCase(qName)) {
							isDescription = false;
						}
					}
				}

				@Override
				public void endDocument() throws SAXException {
				}

				@Override
				public void characters(char[] ch, int start, int length)
						throws SAXException {
					if (isTitle) {
						sbTitle.append(ch, start, length);
					}
					if (isDescription) {
						sbDescription.append(ch, start, length);
					}
				}
			});
			ok = true;
		} catch (Throwable tr) {
			Log.e(TAG, "loadAndParse()", tr);
		}
		description = sbDescription.toString();
		title = sbTitle.toString();
		return ok;
	}
}